#!/usr/bin/env python
"""
Convert a directory with IRC logs to HTML.

Usage: logs2html.py pathname

Needs irclog2html.py.  Produces an index page and a number of HTML-formatted
log files with navigational links.

Looks for *.log in a given directory.  Needs an ISO 8601 date (YYYY-MM-DD or
YYYYMMDD) in the filename.
"""

from __future__ import print_function, unicode_literals

from operator import attrgetter


# Copyright (c) 2005--2013  Marius Gedminas
# latest.log.html symlink suggested by Chris Foster
#
# Released under the terms of the GNU GPL v2 or later
# https://www.gnu.org/copyleft/gpl.html

import os
import re
import sys
import glob
import datetime
import optparse
import shutil

try:
    from urllib import quote
except ImportError:
    from urllib.parse import quote

from . import irclog2html
from .irclog2html import VERSION, RELEASE, HOMEPAGE, escape


# If someone packages this for a Linux distro, they'll want to patch this to
# something like /usr/share/irclog2html/irclog.css, I imagine
CSS_FILE = os.path.join(os.path.dirname(__file__), 'irclog.css')


DATE_REGEXP = re.compile(r'^.*(\d\d\d\d)-?(\d\d)-?(\d\d)')


class Error(Exception):
    """Application error."""


class LogFile:
    """IRC log file."""

    def __init__(self, filename):
        self.filename = filename
        basename = os.path.basename(filename)
        m = DATE_REGEXP.match(basename)
        if not m:
            raise Error("File name does not contain a YYYY-MM-DD date: %s"
                        % filename)
        self.date = datetime.date(*map(int, m.groups()))
        self.link = irclog2html.pick_output_filename(basename)
        self.title = self.date.strftime('%Y-%m-%d (%A)')

    def __eq__(self, other):
        return isinstance(other, LogFile) and other.filename == self.filename

    def __ne__(self, other):
        return not self.__eq__(other)

    def newfile(self):
        """Check whether the log file is new.

        The log file is new if the corresponding html file does not exist
        or was just generated by self.generate.  This implies that navigation
        links for the previous/next file need to be updated.
        """
        if not hasattr(self, '_newfile'):
            # Only do this once, so that self.generate() does not change
            # newness
            self._newfile = not os.path.exists(self.filename + ".html")
        return self._newfile

    def uptodate(self):
        """Check whether the HTML version of the log is up to date."""
        log_mtime = os.stat(self.filename).st_mtime
        try:
            html_mtime = os.stat(self.filename + ".html").st_mtime
        except OSError:
            return False
        return html_mtime > log_mtime

    def generate(self, style, title_prefix='', prev=None, next=None,
                 extra_args=()):
        """Generate HTML for this log file."""
        self.newfile() # update newness flag and remember it
        argv = ['irclog2html', '-s', style]
        argv.extend(extra_args)
        argv += ['-t', title_prefix + self.date.strftime('%A, %Y-%m-%d')]
        if prev:
            argv += ['--prev-url', prev.link,
                     '--prev-title',
                     '&#171; ' + prev.date.strftime('%A, %Y-%m-%d')]
        argv += ['--index-url=index.html', '--index-title=Index']
        if next:
            argv += ['--next-url', next.link,
                     '--next-title',
                     next.date.strftime('%A, %Y-%m-%d') + ' &#187;']
        argv += [self.filename]
        irclog2html.main(argv)


def find_log_files(directory, pattern='*.log'):
    """Find all IRC log files in a given directory.

    Returns a sorted list of LogFile objects (oldest first).
    """
    pattern = os.path.join(directory, pattern)
    # ISO 8601 dates sort the way we need them
    return sorted([
        LogFile(filename)
        for filename in glob.glob(pattern) + glob.glob(pattern + '.gz')
    ], key=attrgetter('filename'))


def write_index(outfile, title, logfiles, searchbox=False, latest_log_link=None):
    """Write an index with links to all log files."""
    print("""\
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
          "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html>
<head>
  <meta http-equiv="Content-Type" content="text/html; charset=%(charset)s" />
  <title>%(title)s</title>
  <link rel="stylesheet" href="irclog.css" />
  <meta name="generator" content="logs2html.py %(VERSION)s by Marius Gedminas" />
  <meta name="version" content="%(VERSION)s - %(RELEASE)s" />
</head>
<body>""" % {'VERSION': VERSION, 'RELEASE': RELEASE,
             'title': escape(title), 'charset': 'UTF-8'}, file=outfile)
    print('<h1>%s</h1>' % escape(title), file=outfile)
    if searchbox:
        print("""
<div class="searchbox">
<form action="search" method="get">
<input type="text" name="q" id="searchtext" />
<input type="submit" value="Search" id="searchbutton" />
</form>
</div>
""", file=outfile)
    if latest_log_link:
        link = escape(quote(latest_log_link))
        print('<ul>', file=outfile)
        print(('<li><a href="%s">Latest (bookmarkable)</a></li>' %
               link), file=outfile)
        print('</ul>', file=outfile)
    print('<ul>', file=outfile)
    current_month = None
    for logfile in logfiles:
        month = logfile.date.strftime('%Y-%m')
        if month != current_month:
            current_month = month
            print('</ul>', file=outfile)
            print('<h2>%s</h2>' % month, file=outfile)
            print('<ul>', file=outfile)
        link = escape(quote(logfile.link))
        title = escape(logfile.title)
        print('<li><a href="%s">%s</a></li>' % (link, title), file=outfile)
    print('</ul>', file=outfile)
    print("""
<div class="generatedby">
<p>Generated by logs2html.py %(VERSION)s by <a href="mailto:marius@pov.lt">Marius Gedminas</a>
 - find it at <a href="%(HOMEPAGE)s">%(HOMEPAGE)s</a>!</p>
</div>
</body>
</html>""" % {'VERSION': VERSION, 'RELEASE': RELEASE,
              'HOMEPAGE': escape(HOMEPAGE)}, file=outfile)


def main(argv=sys.argv):
    progname = 'logs2html'
    parser = optparse.OptionParser("usage: %prog [options] directory",
                                   version=VERSION,
                                   prog=progname,
                                   description="Colourises and converts all IRC"
                                               " logs to HTML format for easy"
                                               " web reading.")
    parser.add_option('-c', '--config', action='callback', type='str',
                      metavar='FILE', callback=irclog2html.do_config_file,
                      help="read options from a config file")
    parser.add_option('-s', '--style', dest="style", default="xhtmltable",
                      help="format log according to specific style"
                           " (default: xhtmltable); passes the style name"
                           " to irclog2html.py")
    parser.add_option('-t', '--title', dest="title", default="IRC logs",
                      help="title of the index page (default: IRC logs)")
    parser.add_option('-p', '--prefix', dest="prefix", default="",
                      help="prefix for page title (e.g.:"
                           " 'IRC logs of #channel for ')")
    parser.add_option('-f', '--force', action="store_true", dest="force",
                      default=False,
                      help="ignore mtime and regenerate all files")
    parser.add_option('-S', '--searchbox', action="store_true", dest="searchbox",
                      default=False,
                      help="include a search box")
    parser.add_option('--dircproxy', action='store_true', default=False,
                      help="dircproxy log file support (strips leading + or - from messages; off by default)")
    parser.add_option('-g', '--glob-pattern', dest="pattern", default="*.log",
                      help="glob pattern that finds log files to be processed"
                      " (default: *.log)")
    parser.add_option('-o', '--output-dir', dest="output_dir", default=None,
                      help="destination output directory"
                           " (default: same as input directory)")
    options, args = parser.parse_args(argv[1:])
    if len(args) < 1:
        parser.error("missing directory name")
    if len(args) > 1:
        parser.error("too many arguments")
    dir = args[0]

    try:
        process(dir, options)
    except Error as e:
        sys.exit("%s: %s" % (progname, e))


def process(dir, options):
    """Process log files in a given directory."""
    extra_args = []
    if options.searchbox:
        extra_args += ['-S']
    if options.dircproxy:
        extra_args += ['--dircproxy']
    out_dir = dir
    if options.output_dir:
        out_dir = options.output_dir
        extra_args += ['--output-file', out_dir]
        if not os.path.isdir(out_dir):
            try:
                os.makedirs(out_dir)
            except OSError as e:
                raise Error("Failed to create directory %s: %s" % (out_dir, e))
    logfiles = find_log_files(dir, options.pattern)
    logfiles.reverse() # newest first
    for n, logfile in enumerate(logfiles):
        if n > 0:
            next = logfiles[n - 1]
        else:
            next = None
        if n + 1 < len(logfiles):
            prev = logfiles[n + 1]
        else:
            prev = None
        if (options.force or not logfile.uptodate()
                or prev and prev.newfile() or next and next.newfile()):
            logfile.generate(options.style, options.prefix, prev, next,
                             extra_args)
    latest_log_link = None
    if logfiles and hasattr(os, "symlink"):
        latest_log_link = 'latest.log.html'
        move_symlink(logfiles[0].link, os.path.join(out_dir, latest_log_link))
    outfilename = os.path.join(out_dir, 'index.html')
    try:
        outfile = open(outfilename, 'w')
    except IOError as e:
        raise Error("cannot open %s for writing: %s" % (outfilename, e))
    try:
        write_index(outfile, options.title, logfiles, options.searchbox,
                    latest_log_link)
    finally:
        outfile.close()
    css_file = os.path.join(out_dir, 'irclog.css')
    if not os.path.exists(css_file) and os.path.exists(CSS_FILE):
        shutil.copy(CSS_FILE, css_file)


def move_symlink(src, dst):
    """Create or overwrite a symlink.

    ``src`` is the link target.

    ``dst`` is the name of the symlink.
    """
    try:
        os.unlink(dst)
    except OSError:
        pass
    os.symlink(src, dst)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
Search IRC logs (a CGI script and a WSGI app).

Expects to find logs matching the IRCLOG_GLOB pattern (default: *.log)
in the directory specified by the IRCLOG_LOCATION environment variable.
Expects the filenames to contain a ISO 8601 date (YYYY-MM-DD).

Apache configuration example:

  ScriptAlias /irclogs/search /path/to/irclogsearch.py
  <Location /irclogs/search>
    SetEnv IRCLOG_LOCATION /path/to/irclog/files/
    # Uncomment the following if your log files use a different format
    #SetEnv IRCLOG_GLOB "*.log.????-??-??"
  </Location>

"""

# Copyright (c) 2006-2013, Marius Gedminas and contributors
#
# Released under the terms of the GNU GPL v2 or later
# https://www.gnu.org/copyleft/gpl.html

from __future__ import print_function, unicode_literals

import cgi
import cgitb
import io
import os
import re
import sys
import time
from contextlib import closing

try:
    from urllib import quote
except ImportError:
    from urllib.parse import quote

from .irclog2html import (LogParser, XHTMLTableStyle, NickColourizer,
                          escape, open_log_file, VERSION, RELEASE, HOMEPAGE)
from .logs2html import find_log_files


try:
    unicode
except NameError:
    # Python 3.x
    unicode = str


DEFAULT_LOGFILE_PATH = os.path.dirname(__file__)
DEFAULT_LOGFILE_PATTERN = "*.log"

DATE_REGEXP = re.compile(r'^.*(\d\d\d\d)-(\d\d)-(\d\d)')


HEADER = """\
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
          "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html>
<head>
  <meta http-equiv="Content-Type" content="text/html; charset=%(charset)s" />
  <title>%(title)s</title>
  <link rel="stylesheet" href="irclog.css" />
  <meta name="generator" content="irclogsearch.py %(VERSION)s by Marius Gedminas" />
  <meta name="version" content="%(VERSION)s - %(RELEASE)s" />
</head>
<body>""" % {'VERSION': VERSION, 'RELEASE': RELEASE,
             'title': escape("Search IRC logs"), 'charset': 'UTF-8'}

FOOTER = """
<div class="generatedby">
<p>Generated by irclogsearch.py %(VERSION)s by <a href="mailto:marius@pov.lt">Marius Gedminas</a>
 - find it at <a href="%(HOMEPAGE)s">%(HOMEPAGE)s</a>!</p>
</div>
</body>
</html>""" % {'VERSION': VERSION, 'RELEASE': RELEASE,
              'HOMEPAGE': escape(HOMEPAGE)}


class Error(Exception):
    """Application error."""


class SearchStats(object):
    """Search statistics."""

    files = 0
    lines = 0
    matches = 0


class SearchResult(object):
    """Search result -- a single utterance."""

    def __init__(self, filename, link, date, time, event, info):
        self.filename = filename
        self.link = link
        self.date = date
        self.time = time
        self.event = event
        self.info = info


class StdoutWrapper(object):
    # Because I can't wrap sys.stdout with io.TextIOWrapper on Python 2

    def __init__(self, stream):
        self.stream = stream
        self.flush = stream.flush
        self.write = stream.write
        self.closed = False

    def readable(self):
        return False

    def writable(self):
        return True

    def seekable(self):
        return False


class SearchResultFormatter(object):
    """Formatter of search results."""

    def __init__(self, stream=None):
        self.stream = stream
        bstream = stream.buffer
        self.style = XHTMLTableStyle(bstream)
        self.nick_colour = NickColourizer()

    def print_prefix(self):
        print(self.style.prefix, file=self.stream)

    def print_html(self, result):
        link = urlescape(result.link)
        if result.event == LogParser.COMMENT:
            nick, text = result.info
            htmlcolour = self.nick_colour[nick]
            self.style.nicktext(result.time, nick, text, htmlcolour, link)
        else:
            if result.event == LogParser.NICKCHANGE:
                text, oldnick, newnick = result.info
                self.nick_colour.change(oldnick, newnick)
            else:
                text = result.info
            self.style.servermsg(result.time, result.event, text, link)

    def print_suffix(self):
        print(self.style.suffix, file=self.stream)


def urlescape(link):
    return escape(quote(link))


def parse_log_file(filename):
    with closing(open_log_file(filename)) as f:
        for row in LogParser(f):
            yield row


def search_irc_logs(query, stats=None, where=DEFAULT_LOGFILE_PATH,
                    logfile_pattern=DEFAULT_LOGFILE_PATTERN, limit=None):
    if not stats:
        stats = SearchStats() # will be discarded, but, oh, well
    query = query.lower()
    files = find_log_files(where, logfile_pattern)
    files.reverse() # newest first
    for f in files:
        date = f.date
        link = f.link
        stats.files += 1
        for timestamp, event, info in parse_log_file(f.filename):
            if event == LogParser.COMMENT:
                nick, text = info
                text = nick + ' ' + text
            elif event == LogParser.NICKCHANGE:
                text, oldnick, newnick = info
            else:
                text = unicode(info)
            stats.lines += 1
            if query in text.lower():
                stats.matches += 1
                yield SearchResult(f.filename, link, date, timestamp, event, info)
                if stats.matches == limit:
                    return


def print_cgi_headers(stream):
    print("Content-Type: text/html; charset=UTF-8", file=stream)
    print("", file=stream)


def print_search_form(stream=None):
    if stream is None:
        stream = sys.stdout
    print(HEADER, file=stream)
    print("<h1>Search IRC logs</h1>", file=stream)
    print('<form action="" method="get">', file=stream)
    print('<input type="text" name="q" />', file=stream)
    print('<input type="submit" />', file=stream)
    print('</form>', file=stream)
    print(FOOTER, file=stream)


def print_search_results(query, where=DEFAULT_LOGFILE_PATH,
                         logfile_pattern=DEFAULT_LOGFILE_PATTERN,
                         limit=100,
                         stream=None):
    if stream is None:
        stream = sys.stdout
    print(HEADER, file=stream)
    print("<h1>IRC log search results for %s</h1>" % escape(query), file=stream)
    print('<form action="" method="get">', file=stream)
    print('<input type="text" name="q" value="%s" />' % escape(query),
          file=stream)
    print('<input type="submit" />', file=stream)
    print('</form>', file=stream)
    started = time.time()
    date = None
    prev_result = None
    formatter = SearchResultFormatter(stream)
    stats = SearchStats()
    for result in search_irc_logs(query, stats=stats, where=where,
                                  logfile_pattern=logfile_pattern,
                                  limit=limit):
        if date != result.date:
            if prev_result:
                formatter.print_suffix()
                prev_result = None
            if date:
                print("  </li>", file=stream)
            else:
                print('<ul class="searchresults">', file=stream)
            print('  <li><a href="%s">%s</a>:' %
                  (urlescape(result.link),
                   result.date.strftime('%Y-%m-%d (%A)')),
                  file=stream)
            date = result.date
        if not prev_result:
            formatter.print_prefix()
        formatter.print_html(result)
        prev_result = result
    if prev_result:
        formatter.print_suffix()
    if date:
        print("  </li>", file=stream)
        print("</ul>", file=stream)
    total_time = time.time() - started
    print("<p>%d matches in %d log files with %d lines (%.1f seconds).</p>"
          % (stats.matches, stats.files, stats.lines, total_time),
          file=stream)
    print(FOOTER, file=stream)


def unicode_stdout():
    if hasattr(sys.stdout, 'buffer'):
        stream = sys.stdout.buffer # Python 3
    else:
        stream = StdoutWrapper(sys.stdout) # Python 2
    return io.TextIOWrapper(stream, 'ascii',
                            errors='xmlcharrefreplace',
                            line_buffering=True)


def search_page(stream, form, where, logfile_pattern):
    if "q" not in form:
        print_search_form(stream)
    else:
        search_text = form["q"].value
        if isinstance(search_text, bytes):
            search_text = search_text.decode('UTF-8')
        print_search_results(search_text, stream=stream, where=where,
                             logfile_pattern=logfile_pattern)


def main():
    """CGI script"""
    cgitb.enable()
    logfile_path = os.getenv('IRCLOG_LOCATION') or DEFAULT_LOGFILE_PATH
    logfile_pattern = os.getenv('IRCLOG_GLOB') or DEFAULT_LOGFILE_PATTERN
    form = cgi.FieldStorage()
    stream = unicode_stdout()
    print_cgi_headers(stream)
    search_page(stream, form, logfile_path, logfile_pattern)


if __name__ == '__main__':
    main()



import doctest


# Just test that there are no syntax errors and main() can imported; there's
# no actual code in __main__.py.


def doctest_main_can_be_imported():
    """Test for __main__.py

        >>> from irclog2html.__main__ import main  # noqa

    """


def test_suite():
    optionflags = doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS | doctest.REPORT_NDIFF
    return doctest.DocTestSuite(optionflags=optionflags)



# -*- coding: utf-8 -*-
import datetime
import gzip
import io
import os
import shutil
import tempfile
import doctest
import unittest
from contextlib import closing

import mock

from irclog2html.irclogserver import dir_listing, parse_path, application


here = os.path.dirname(__file__)


def gzip_copy(src, dst):
    with open(src, 'rb') as fi:
        with closing(gzip.open(dst, 'wb')) as fo:
            shutil.copyfileobj(fi, fo)


def set_up_sample():
    tmpdir = tempfile.mkdtemp(prefix='irclog2html-test-')
    os.mkdir(os.path.join(tmpdir, '.hidden'))
    gzip_copy(os.path.join(here, 'sample.log'),
              os.path.join(tmpdir, 'sample-2013-03-17.log.gz'))
    shutil.copy(os.path.join(here, 'sample.log'),
                os.path.join(tmpdir, 'sample-2013-03-18.log'))
    with open(os.path.join(tmpdir, "index.html"), "w") as f:
        f.write("This is the index")
    with open(os.path.join(tmpdir, "font.css"), "w") as f:
        f.write("* { font: comic sans; }")
    os.mkdir(os.path.join(tmpdir, "#chan"))
    with open(os.path.join(tmpdir, "#chan", "index.html"), "w") as f:
        f.write("#chan index")
    shutil.copy(os.path.join(here, 'sample.log'),
                os.path.join(tmpdir, '#chan', 'sample-2013-03-18.log'))
    return tmpdir


def clean_up_sample(tmpdir):
    shutil.rmtree(tmpdir)


def doctest_parse_path():
    """Test for parse_path.

    This function decides whether to search or to display a file based
    on URL path:

        >>> parse_path(dict(PATH_INFO='/search'))
        (None, 'search')

        >>> parse_path(dict(PATH_INFO='/#channel-2015-05-05.log.html'))
        (None, '#channel-2015-05-05.log.html')

    When there is no file name, we show the index:

        >>> parse_path(dict(PATH_INFO='/'))
        (None, 'index.html')

    Any slashes other than the leading one result in None:

        >>> parse_path(dict(PATH_INFO='/../../etc/passwd'))
        (None, None)

    But there is an option to serve a directory with subdir for each
    channel.  If IRCLOG_CHAN_DIR is defined, the first traversal step
    is the first element of the returned tuple:

        >>> parse_path(dict(PATH_INFO='/#random/search',
        ...            IRCLOG_CHAN_DIR='/opt/irclog'))
        ('#random', 'search')


        >>> parse_path(dict(PATH_INFO='/#random/',
        ...            IRCLOG_CHAN_DIR='/opt/irclog'))
        ('#random', 'index.html')

    If the path does not contain the channel name, tough cookies:

        >>> parse_path(dict(PATH_INFO='/index.html',
        ...            IRCLOG_CHAN_DIR='/opt/irclog'))
        (None, 'index.html')

    Hacking verboten:

        >>> parse_path(dict(PATH_INFO='/../index.html',
        ...            IRCLOG_CHAN_DIR='/opt/irclog'))
        (None, None)

        >>> parse_path(dict(PATH_INFO='/#random/../index.html',
        ...            IRCLOG_CHAN_DIR='/opt/irclog'))
        (None, None)

    """


class TestDirListing(unittest.TestCase):

    def make_channel(self, name, age):
        m = mock.Mock(age=age)  # can't pass name here :(
        m.name = name
        return m

    @mock.patch('irclog2html.irclogserver.find_channels')
    def test_dir_listing_old_an_new(self, mock_find_channels):
        mock_find_channels.return_value = [
            self.make_channel(name='#cobwebs', age=datetime.timedelta(days=7.5)),
            self.make_channel(name='#rainbows', age=datetime.timedelta(minutes=5)),
            self.make_channel(name='#puppies', age=datetime.timedelta(days=6.5)),
        ]
        stream = io.StringIO()
        dir_listing(stream, '/all/my/logs')
        response = stream.getvalue()
        self.assertIn('<h2>Active channels</h2>', response)
        self.assertIn('#rainbows', response)
        self.assertIn('#puppies', response)
        self.assertIn('<h2>Old channels</h2>', response)
        self.assertIn('#cobwebs', response)
        self.assertTrue(
            response.index('Active channels') <
            response.index('#rainbows') <
            response.index('#puppies') <
            response.index('Old channels') <
            response.index('#cobwebs')
        )

    @mock.patch('irclog2html.irclogserver.find_channels')
    def test_dir_listing_old(self, mock_find_channels):
        mock_find_channels.return_value = [
            self.make_channel(name='#cobwebs', age=datetime.timedelta(days=7.5)),
        ]
        stream = io.StringIO()
        dir_listing(stream, '/all/my/logs')
        response = stream.getvalue()
        self.assertNotIn('<h2>Active channels</h2>', response)
        self.assertNotIn('<h2>Old channels</h2>', response)
        self.assertIn('#cobwebs', response)

    @mock.patch('irclog2html.irclogserver.find_channels')
    def test_dir_listing_new(self, mock_find_channels):
        mock_find_channels.return_value = [
            self.make_channel(name='#rainbows', age=datetime.timedelta(minutes=5)),
            self.make_channel(name='#puppies', age=datetime.timedelta(days=6.5)),
        ]
        stream = io.StringIO()
        dir_listing(stream, '/all/my/logs')
        response = stream.getvalue()
        self.assertNotIn('<h2>Active channels</h2>', response)
        self.assertNotIn('<h2>Old channels</h2>', response)
        self.assertIn('#rainbows', response)
        self.assertIn('#puppies', response)

    @mock.patch('irclog2html.irclogserver.find_channels')
    def test_dir_listing_empty(self, mock_find_channels):
        mock_find_channels.return_value = []
        stream = io.StringIO()
        dir_listing(stream, '/all/my/logs')
        response = stream.getvalue()
        self.assertNotIn('<h2>Active channels</h2>', response)
        self.assertNotIn('<h2>Old channels</h2>', response)
        self.assertIn('<p>No channels found.</p>', response)


class Response(object):
    pass


class TestApplication(unittest.TestCase):

    def setUp(self):
        self.tmpdir = set_up_sample()

    def tearDown(self):
        clean_up_sample(self.tmpdir)

    def request(self, path='/', expect=200, extra_env=None):
        environ = {
            'IRCLOG_LOCATION': self.tmpdir,
            'PATH_INFO': path.partition('?')[0],
            'QUERY_STRING': path.partition('?')[-1],
            'wsgi.input': None,
        }
        if extra_env:
            environ.update(extra_env)
        start_response = mock.Mock()
        response = Response()
        response.body = b''.join(application(environ, start_response))
        self.assertEqual(start_response.call_count, 1)
        status, headers = start_response.call_args[0]
        response.status_string = status
        response.status = int(status.split()[0])
        response.header_list = headers
        response.headers = dict(headers)
        response.content_type = response.headers['Content-Type']
        response.location = response.headers.get('Location')
        self.assertEqual(response.status, expect)
        return response

    def test_root(self):
        response = self.request('/')
        self.assertEqual(response.body, b'This is the index')
        self.assertEqual(response.content_type, 'text/html; charset=UTF-8')

    def test_root_without_index_html(self):
        os.unlink(os.path.join(self.tmpdir, 'index.html'))
        response = self.request('/')
        self.assertEqual(response.content_type, 'text/html; charset=UTF-8')
        self.assertIn(b'<title>IRC logs</title>', response.body)
        self.assertIn(b'<a href="sample-2013-03-18.log.html">', response.body)

    def test_search_page(self):
        response = self.request('/search')
        self.assertEqual(response.content_type, 'text/html; charset=UTF-8')
        self.assertIn(b'<title>Search IRC logs</title>', response.body)

    def test_search(self):
        response = self.request('/search?q=bot')
        self.assertEqual(response.content_type, 'text/html; charset=UTF-8')
        self.assertIn(b'<title>Search IRC logs</title>', response.body)
        self.assertIn(b'<p>10 matches in 2 log files with 20 lines',
                      response.body)

    def test_log_file(self):
        response = self.request('/sample-2013-03-18.log')
        self.assertEqual(response.content_type, 'text/plain; charset=UTF-8')
        self.assertIn(b'2005-01-08T23:33:54  *** povbot has joined #pov',
                      response.body)
        self.assertIn(u'ąčę'.encode('UTF-8'), response.body)
        self.assertIn(u'<mgedmin> š'.encode('UTF-8'), response.body)

    def test_dynamic_log_file_html(self):
        response = self.request('/sample-2013-03-18.log.html')
        self.assertEqual(response.content_type, 'text/html; charset=UTF-8')
        self.assertIn(
            b'<title>IRC log for Monday, 2013-03-18</title>',
            response.body)
        self.assertIn(
            b'<td class="join" colspan="2">*** povbot has joined #pov</td>',
            response.body)
        self.assertIn(u'ąčę'.encode('UTF-8'), response.body)
        self.assertIn(u'š'.encode('UTF-8'), response.body)

    def test_builtin_css(self):
        response = self.request('/irclog.css')
        self.assertEqual(response.content_type, 'text/css')
        self.assertIn(b'div.searchbox {', response.body)

    @mock.patch('irclog2html.irclogserver.CSS_FILE', '/nosuchfile')
    def test_builtin_css_missing(self):
        response = self.request('/irclog.css', expect=404)
        self.assertEqual(response.content_type, 'text/plain')
        self.assertIn(b'Not found', response.body)

    def test_other_css(self):
        response = self.request('/font.css')
        self.assertEqual(response.content_type, 'text/css')
        self.assertIn(b'{ font: comic sans; }', response.body)

    def test_not_found(self):
        response = self.request('/nosuchfile', expect=404)
        self.assertEqual(response.content_type, 'text/plain')
        self.assertIn(b'Not found', response.body)

    def test_html_not_found(self):
        response = self.request('/nosuchfile.html', expect=404)
        self.assertEqual(response.content_type, 'text/plain')
        self.assertIn(b'Not found', response.body)

    def test_html_not_found_stupid_corner_case(self):
        response = self.request('/2016-09-25.html', expect=404)
        self.assertEqual(response.content_type, 'text/plain')
        self.assertIn(b'Not found', response.body)

    def test_path_with_slashes(self):
        response = self.request('/./index.html', expect=404)
        self.assertEqual(response.content_type, 'text/plain')
        self.assertIn(b'Not found', response.body)

    def test_path_with_backslashes(self):
        response = self.request('/.\\index.html', expect=404)
        self.assertEqual(response.content_type, 'text/plain')
        self.assertIn(b'Not found', response.body)

    def test_chan_index(self):
        response = self.request(
            '/#chan/',
            extra_env={"IRCLOG_CHAN_DIR": self.tmpdir})
        self.assertEqual(response.content_type, 'text/html; charset=UTF-8')
        self.assertEqual(b'#chan index', response.body)

    def test_chan_index_no_trailing_slash(self):
        response = self.request(
            '/#chan',
            extra_env={"IRCLOG_CHAN_DIR": self.tmpdir},
            expect=302)
        self.assertEqual(response.location, '%23chan/')

    def test_chan_index_without_index_html(self):
        os.unlink(os.path.join(self.tmpdir, '#chan', 'index.html'))
        response = self.request(
            '/#chan/',
            extra_env={"IRCLOG_CHAN_DIR": self.tmpdir})
        self.assertEqual(response.content_type, 'text/html; charset=UTF-8')
        self.assertIn(b'<title>IRC logs of #chan</title>', response.body)

    def test_chan_search_page(self):
        response = self.request(
            '/#chan/search',
            extra_env={"IRCLOG_CHAN_DIR": self.tmpdir})
        self.assertEqual(response.content_type, 'text/html; charset=UTF-8')
        self.assertIn(b'<title>Search IRC logs</title>', response.body)

    def test_chan_listing(self):
        response = self.request(
            '/',
            extra_env={"IRCLOG_CHAN_DIR": self.tmpdir})
        self.assertEqual(response.content_type, 'text/html; charset=UTF-8')
        self.assertIn(b'IRC logs', response.body)
        self.assertIn(b'<a href="%23chan/">#chan</a>', response.body)
        self.assertNotIn(b'.hidden', response.body)

    def test_chan_error(self):
        response = self.request(
            '/../index.html',
            extra_env={"IRCLOG_CHAN_DIR": self.tmpdir},
            expect=404)
        self.assertEqual(response.content_type, 'text/plain')
        self.assertIn(b'Not found', response.body)

    def test_chan_dynamic_log_file_html(self):
        response = self.request('/#chan/sample-2013-03-18.log.html',
                                extra_env={"IRCLOG_CHAN_DIR": self.tmpdir})
        self.assertEqual(response.content_type, 'text/html; charset=UTF-8')
        self.assertIn(
            b'<title>IRC log of #chan for Monday, 2013-03-18</title>',
            response.body)
        self.assertIn(
            b'<td class="join" colspan="2">*** povbot has joined #pov</td>',
            response.body)

    @mock.patch("os.environ")
    def test_chan_os_environ(self, environ):
        os.environ.get = {"IRCLOG_CHAN_DIR": self.tmpdir}.get
        response = self.request('/')
        self.assertEqual(response.content_type, 'text/html; charset=UTF-8')
        self.assertIn(b'IRC logs', response.body)
        self.assertIn(b'<a href="%23chan/">#chan</a>', response.body)

    @mock.patch("os.environ")
    def test_chan_search_page_os_environ(self, environ):
        os.environ.get = {"IRCLOG_CHAN_DIR": self.tmpdir}.get
        response = self.request(
            '/#chan/search')
        self.assertEqual(response.content_type, 'text/html; charset=UTF-8')
        self.assertIn(b'<title>Search IRC logs</title>', response.body)


def test_suite():
    return unittest.TestSuite([
        unittest.defaultTestLoader.loadTestsFromName(__name__),
        doctest.DocTestSuite()
    ])



#!/usr/bin/env python
"""
Serve IRC logs (WSGI app)

Expects to find logs matching the IRCLOG_GLOB pattern (default: *.log)
in the directory specified by the IRCLOG_LOCATION environment variable.
Expects the filenames to contain a ISO 8601 date (YYYY-MM-DD).

Apache configuration example:

  WSGIScriptAlias /irclogs /path/to/irclogserver.py
  <Location /irclogs>
    # If you're serving the logs for one channel, specify this:
    SetEnv IRCLOG_LOCATION /path/to/irclog/files/
    # If you're serving the logs for many channels, specify this:
    SetEnv IRCLOG_CHAN_DIR /path/to/irclog/channels/
    # Uncomment the following if your log files use a different format
    #SetEnv IRCLOG_GLOB "*.log.????-??-??"
  </Location>

"""

# Copyright (c) 2015-2016, Marius Gedminas and contributors
#
# Released under the terms of the GNU GPL v2 or later
# https://www.gnu.org/copyleft/gpl.html

from __future__ import print_function

import argparse
import cgi
import datetime
import io
import os
import time
from operator import attrgetter
from wsgiref.simple_server import make_server

try:
    from urllib import quote_plus # Py2
except ImportError:
    from urllib.parse import quote_plus # Py3

from ._version import __version__, __date__
from .irclog2html import (
    CSS_FILE, LogParser, XHTMLTableStyle, convert_irc_log, escape,
)
from .logs2html import LogFile, Error, find_log_files, write_index
from .irclogsearch import (
    DEFAULT_LOGFILE_PATH, DEFAULT_LOGFILE_PATTERN, search_page,
)


HEADER = u'''\
<!DOCTYPE html>
<html>
<head>
  <meta charset="UTF-8">
  <title>{title}</title>
  <link rel="stylesheet" href="irclog.css">
  <meta name="generator" content="irclogserver.py {version} by Marius Gedminas">
  <meta name="version" content="{version} - {date}">
</head>
<body>'''.format(title=u'IRC logs', version=__version__, date=__date__)


FOOTER = u'''
<div class="generatedby">
<p>Generated by irclogserver.py {version} by <a href="mailto:marius@pov.lt">Marius Gedminas</a>
 - find it at <a href="https://mg.pov.lt/irclog2html/">mg.pov.lt</a>!</p>
</div>
</body>
</html>'''.format(version=__version__, date=__date__)


class Channel(object):
    """IRC channel."""

    def __init__(self, name, path):
        self.name = name
        self.mtime = os.stat(os.path.join(path, name)).st_mtime

    @property
    def age(self):
        return datetime.timedelta(seconds=time.time() - self.mtime)


def find_channels(path):
    return sorted([
        Channel(name, path) for name in os.listdir(path)
        if not name.startswith('.') and os.path.isdir(os.path.join(path, name))
    ], key=attrgetter('name'))


def dir_listing(stream, path):
    """Primitive listing of subdirectories."""
    print(HEADER, file=stream)
    print(u"<h1>IRC logs</h1>", file=stream)
    channels = find_channels(path)
    old, new = [], []
    for channel in channels:
        if channel.age > datetime.timedelta(days=7):
            old.append(channel)
        else:
            new.append(channel)

    if not channels:
        print(u"<p>No channels found.</p>", file=stream)

    if new:
        if old:
            print(u'<h2>Active channels</h2>', file=stream)
        print(u"<ul>", file=stream)
        for channel in new:
            print(u'<li><a href="%s/">%s</a></li>'
                  % (quote_plus(channel.name), escape(channel.name)),
                  file=stream)
        print(u"</ul>", file=stream)

    if old:
        if new:
            print(u'<h2>Old channels</h2>', file=stream)
        print(u"<ul>", file=stream)
        for channel in old:
            print(u'<li><a href="%s/">%s</a></li>'
                  % (quote_plus(channel.name), escape(channel.name)),
                  file=stream)
        print(u"</ul>", file=stream)

    print(FOOTER, file=stream)


def log_listing(stream, path, pattern, channel=None):
    """Primitive listing of log files."""
    logfiles = find_log_files(path, pattern)
    logfiles.reverse()
    if channel:
        title = u"IRC logs of {channel}".format(channel=channel)
    else:
        title = u"IRC logs"
    write_index(stream, title, logfiles, searchbox=True)


def dynamic_log(stream, path, pattern, channel=None):
    """Render HTML dynamically"""
    lf = LogFile(path)
    logfiles = find_log_files(os.path.dirname(path), pattern)
    try:
        idx = logfiles.index(lf)
        lf.prev = logfiles[idx - 1] if idx > 0 else None
        lf.next = logfiles[idx + 1] if idx + 1 < len(logfiles) else None
    except ValueError:
        pass
    with open(path, 'rb') as f:
        parser = LogParser(f)
        formatter = XHTMLTableStyle(stream.buffer)
        if channel:
            title = u"IRC log of {channel}".format(channel=channel)
        else:
            title = u"IRC log"
        title += u" for {date:%A, %Y-%m-%d}".format(date=lf.date)
        prev = ('&#171; {date:%A, %Y-%m-%d}'.format(date=lf.prev.date),
                lf.prev.link) if lf.prev else ('', '')
        next = ('{date:%A, %Y-%m-%d} &#187;'.format(date=lf.next.date),
                lf.next.link) if lf.next else ('', '')
        index = ('Index', 'index.html')
        convert_irc_log(parser, formatter, title, prev, index, next,
                        searchbox=True)


def parse_path(environ):
    """Return tuples (channel, filename).

    The channel of None means default, the filename of None means 404.
    """
    path = environ.get('PATH_INFO', '/')
    path = path[1:]  # Remove the leading slash
    channel = None
    if environ.get('IRCLOG_CHAN_DIR', os.environ.get('IRCLOG_CHAN_DIR')):
        if '/' in path:
            channel, path = path.split('/', 1)
    if channel == '..' or path == '..' or '/' in path or '\\' in path:
        return None, None
    return channel, (path or 'index.html')


def application(environ, start_response):
    """WSGI application"""
    def getenv(name, default=None):
        return environ.get(name, os.environ.get(name, default))

    chan_path = getenv('IRCLOG_CHAN_DIR')
    logfile_path = getenv('IRCLOG_LOCATION') or DEFAULT_LOGFILE_PATH
    logfile_pattern = getenv('IRCLOG_GLOB') or DEFAULT_LOGFILE_PATTERN
    form = cgi.FieldStorage(fp=environ['wsgi.input'], environ=environ)
    stream = io.TextIOWrapper(io.BytesIO(), 'ascii',
                              errors='xmlcharrefreplace',
                              line_buffering=True)

    status = "200 Ok"
    content_type = "text/html; charset=UTF-8"
    headers = {}
    result = []

    channel, path = parse_path(environ)
    if channel:
        logfile_path = os.path.join(chan_path, channel)
    if path is None:
        status = "404 Not Found"
        result = [b"Not found"]
        content_type = "text/plain"
    elif path == "index.html" and chan_path and channel is None:
        dir_listing(stream, chan_path)
        result = [stream.buffer.getvalue()]
    elif path == 'search':
        search_page(stream, form, logfile_path, logfile_pattern)
        result = [stream.buffer.getvalue()]
    elif path == 'irclog.css':
        content_type = "text/css"
        try:
            with open(CSS_FILE, "rb") as f:
                result = [f.read()]
        except IOError:  # pragma: nocover
            status = "404 Not Found"
            result = [b"Not found"]
            content_type = "text/plain"
    else:
        full_path = os.path.join(logfile_path, path)
        try:
            with open(full_path, "rb") as f:
                result = [f.read()]
        except IOError:
            if path == 'index.html':
                log_listing(stream, logfile_path, logfile_pattern, channel)
                result = [stream.buffer.getvalue()]
            elif path.endswith('.html'):
                try:
                    dynamic_log(stream, full_path[:-len('.html')],
                                logfile_pattern, channel=channel)
                    result = [stream.buffer.getvalue()]
                except (Error, IOError):
                    # Error will be raised if the filename has no ISO-8601 date
                    status = "404 Not Found"
                    result = [b"Not found"]
                    content_type = "text/plain"
            elif chan_path and not channel and os.path.isdir(os.path.join(chan_path, path)):
                status = "302 Found"
                headers["Location"] = quote_plus(path) + "/"
                result = [b"Redirecting..."]
                content_type = "text/plain"
            else:
                status = "404 Not Found"
                result = [b"Not found"]
                content_type = "text/plain"
        else:
            if path.endswith('.css'):
                content_type = "text/css"
            elif path.endswith('.log') or path.endswith('.txt'):
                content_type = "text/plain; charset=UTF-8"
                result = [LogParser.decode(line).encode('UTF-8')
                          for line in b''.join(result).splitlines(True)]

    headers["Content-Type"] = content_type
    # We need str() for Python 2 because of unicode_literals
    headers = sorted((str(k), str(v)) for k, v in headers.items())
    start_response(str(status), headers)
    return result


def main():  # pragma: nocover
    """Simple web server for manual testing"""
    parser = argparse.ArgumentParser(description="Serve IRC logs")
    parser.add_argument(
        '-p', '--port', type=int, default=8080,
        help='listen on the specified port (default: 8080)')
    parser.add_argument(
        '-P', '--pattern',
        help='IRC log file pattern (default: $IRCLOG_GLOB,'
             ' falling back to %s)' % DEFAULT_LOGFILE_PATTERN)
    parser.add_argument(
        '-m', '--multi', action='store_true',
        help='serve logs for multiple channels in subdirectories'
             ' (default: when $IRCLOG_CHAN_DIR points to a path)')
    parser.add_argument(
        'path',
        help='where to find IRC logs (default: $IRCLOG_LOCATION'
             ' or $IRCLOG_CHAN_DIR, falling back to %s)'
             % DEFAULT_LOGFILE_PATH)
    args = parser.parse_args()
    srv = make_server('localhost', args.port, application)
    print("Started at http://localhost:{port}/".format(port=args.port))
    if args.multi:
        os.environ['IRCLOG_CHAN_DIR'] = args.path
        print("Serving IRC logs for multiple channels from {path}".format(
            path=args.path))
    else:
        os.environ['IRCLOG_LOCATION'] = args.path
        print("Serving IRC logs from {path}".format(path=args.path))
    if args.pattern:
        os.environ['IRCLOG_GLOB'] = args.pattern
        print("Looking for files matching {pattern}".format(
            pattern=args.pattern))
    try:
        srv.serve_forever()
    except KeyboardInterrupt:
        pass


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
Functional tests for irclog2html.py

You must run this script in a directory that contains irclog2html.py,
irclog2html.pl and testcases/*.log.  Both scripts must be executable.
"""
import os
import tempfile
import shutil
import difflib
import glob

from irclog2html import VERSION, RELEASE


def replace(s, replacements):
    """Replace a bunch of things in a string."""
    replacements = [(-len(k), k, v) for k, v in replacements.iteritems()]
    replacements.sort() # longest first
    for sortkey, k, v in replacements:
        s = s.replace(k, v)
    return s


def run_in_tempdir(inputfile, script, args):
    """Copy inputfile into a temporary directory and run a given command.

    Arguments for the script are args with the name of the copied file
    appended (e.g. run('/var/log/irc.log', 'irclog2html.py', '-s table')
    will run irclog2html.py -s table /tmp/tempdirname/irc.log)

    Performs no shell escaping whatsoever.

    Returns the contents of the output file (inputfile + '.html' in the
    temporary directory).
    """
    dir = tempfile.mkdtemp()
    try:
        newinputfile = os.path.join(dir, os.path.basename(inputfile))
        shutil.copyfile(inputfile, newinputfile)
        cmdline = '%s %s %s' % (script, args, newinputfile)
        pipe = os.popen('%s 2>&1' % cmdline, 'r')
        output = pipe.read()
        status = pipe.close()
        if status:
            raise AssertionError('%s returned status code %s\n%s'
                                 % (cmdline, status, output))
        if output:
            raise AssertionError('%s said\n%s'
                                 % (cmdline, output))
        outfilename = newinputfile + '.html'
        try:
            return open(outfilename).read().replace(dir, '/tmpdir')
        except IOError, e:
            raise AssertionError('%s did not create the output file\n%s' %
                                 (cmdline, e))
    finally:
        shutil.rmtree(dir)


def run_and_compare(inputfile, args=""):
    """Run irclog2html.pl and irclog2html.py on inputfile and compare outputs.

    args specify additional command line arguments.
    """
    output1 = run_in_tempdir(inputfile, './irclog2html.py', args)
    output1 = replace(output1, {'irclog2html.py': 'SCRIPT',
                                'Marius Gedminas': 'AUTHOR',
                                VERSION: 'VERSION',
                                RELEASE: 'REVISION',
                                'marius@pov.lt': 'EMAIL',
                                'https://mg.pov.lt/irclog2html/': 'URL',
                                'mg.pov.lt': 'WEBSITE'})
    output2 = run_in_tempdir(inputfile, './irclog2html.pl', args)
    output2 = replace(output2, {'irclog2html.pl': 'SCRIPT',
                                'Jeff Waugh': 'AUTHOR',
                                '2.1mg': 'VERSION',
                                '27th July, 2001': 'REVISION',
                                'jdub@NOSPAMperkypants.org': 'EMAIL',
                                'http://freshmeat.net/projects/irclog2html.pl/':
                                    'URL',
                                'freshmeat.net': 'WEBSITE'})
    if output1 != output2:
        raise AssertionError('files differ (- irclog2html.py,'
                             ' + irclog2html.pl):\n' +
                             ''.join(difflib.ndiff(output1.splitlines(True),
                                                   output2.splitlines(True))))


DEFAULT_ARGS = ('-s table', '-s simplett', '-s tt', '-s simpletable',
                '-s table --colour-part="#deadbe"',
                '-s table --color-action=#cafeba')


def testcase(inputfile, args_to_try=DEFAULT_ARGS):
    """Run both scripts on inputfile with various arguments."""
    print inputfile,
    try:
        for args in args_to_try:
            print ".",
            run_and_compare(inputfile, args)
    except AssertionError, e:
        print "FAILED"
        print
        print e
        print
    else:
        print "ok"


def main():
    os.chdir(os.path.dirname(__file__))
    # the Perl script takes ages to process dircproxy-example.log; ignore it
    testcases = glob.glob('testcases/test*.log')
    print "Comparing outputs produced by the Perl version and the Python port"
    print "There are %d test cases" % len(testcases)
    n = 0
    for inputfile in testcases:
        n += 1
        print n,
        testcase(inputfile)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
import os
import re
from io import open
from setuptools import setup


here = os.path.dirname(__file__)


def read(filename):
    with open(os.path.join(here, filename), encoding='utf-8') as f:
        return f.read()


long_description = read('README.rst') + '\n\n' + read('CHANGES.rst')

version_file = os.path.join(here, 'src/irclog2html/_version.py')
d = dict(re.findall('''(__[a-z]+__) *= *'([^']*)''', read(version_file)))
version = d['__version__']
homepage = d['__homepage__']

setup(
    name='irclog2html',
    version=version,
    author='Marius Gedminas',
    author_email='marius@gedmin.as',
    license='GPL v2 or later',
    platforms=['any'],
    url=homepage,
    description='Convert IRC logs to HTML',
    long_description=long_description,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
    ],
    keywords='irc log colorizer html wsgi',
    extras_require=dict(test=[
        "mock",
        "zope.testing",
    ]),
    packages=['irclog2html'],
    package_dir={'': 'src'},
    include_package_data=True,
    entry_points="""
        [console_scripts]
        irclog2html = irclog2html.irclog2html:main
        logs2html = irclog2html.logs2html:main
        irclogsearch = irclog2html.irclogsearch:main
        irclogserver = irclog2html.irclogserver:main
    """,
    zip_safe=False,
)
